    PROGRAM f08bffe

!      F08BFF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgeqp3, dnrm2, dormqr, dtrsm, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       REAL (KIND=nag_wp), PARAMETER   :: one = 1.0E0_nag_wp
       REAL (KIND=nag_wp), PARAMETER   :: zero = 0.0E0_nag_wp
       INTEGER, PARAMETER              :: inc1 = 1, nb = 64, nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: tol
       INTEGER                         :: i, ifail, info, j, k, lda, ldb,      &
                                          lwork, m, n, nrhs
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), b(:,:), rnorm(:), tau(:),    &
                                          work(:)
       INTEGER, ALLOCATABLE            :: jpvt(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          abs
!      .. Executable Statements ..
       WRITE (nout,*) 'F08BFF Example Program Results'
       WRITE (nout,*)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) m, n, nrhs
       lda = m
       ldb = m
       lwork = 2*n + (n+1)*nb
       ALLOCATE (a(lda,n),b(ldb,nrhs),rnorm(n),tau(n),work(lwork),jpvt(n))

!      Read A and B from data file

       READ (nin,*) (a(i,1:n),i=1,m)
       READ (nin,*) (b(i,1:nrhs),i=1,m)

!      Initialize JPVT to be zero so that all columns are free

       jpvt(1:n) = 0

!      Compute the QR factorization of A
!      The NAG name equivalent of dgeqp3 is f08bff
       CALL dgeqp3(m,n,a,lda,jpvt,tau,work,lwork,info)

!      Compute C = (C1) = (Q**T)*B, storing the result in B
!                   (C2)
!      The NAG name equivalent of dormqr is f08agf
       CALL dormqr('Left','Transpose',m,nrhs,n,a,lda,tau,b,ldb,work,lwork, &
          info)

!      Choose TOL to reflect the relative accuracy of the input data

       tol = 0.01_nag_wp

!      Determine and print the rank, K, of R relative to TOL

LOOP:  DO k = 1, n
          IF (abs(a(k,k))<=tol*abs(a(1,1))) EXIT LOOP
       END DO LOOP
       k = k - 1

       WRITE (nout,*) 'Tolerance used to estimate the rank of A'
       WRITE (nout,99999) tol
       WRITE (nout,*) 'Estimated rank of A'
       WRITE (nout,99998) k
       WRITE (nout,*)
       FLUSH (nout)

!      Compute least-squares solutions by backsubstitution in
!      R(1:K,1:K)*Y = C1, storing the result in B

       CALL dtrsm('Left','Upper','No transpose','Non-Unit',k,nrhs,one,a,lda,b, &
          ldb)

!      Compute estimates of the square roots of the residual sums of
!      squares (2-norm of each of the columns of C2)

!      The NAG name equivalent of dnrm2 is f06ejf
       DO j = 1, nrhs
          rnorm(j) = dnrm2(m-k,b(k+1,j),inc1)
       END DO

!      Set the remaining elements of the solutions to zero (to give
!      the basic solutions)

       b(k+1:n,1:nrhs) = zero

!      Permute the least-squares solutions stored in B to give X = P*Y

       DO j = 1, nrhs
          work(jpvt(1:n)) = b(1:n,j)
          b(1:n,j) = work(1:n)
       END DO

!      Print least-squares solutions

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,nrhs,b,ldb,'Least-squares solution(s)', &
          ifail)

!      Print the square roots of the residual sums of squares

       WRITE (nout,*)
       WRITE (nout,*) 'Square root(s) of the residual sum(s) of squares'
       WRITE (nout,99999) rnorm(1:nrhs)

99999  FORMAT (5X,1P,6E11.2)
99998  FORMAT (1X,I8)
    END PROGRAM f08bffe
