    PROGRAM f08nffe

!      F08NFF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgehrd, dgemm, dhseqr, dlange => f06raf, &
                               dorghr, nag_wp, x02ajf, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: alpha, beta, norm
       INTEGER                         :: i, ifail, info, lda, ldc, ldd, ldz,  &
                                          lwork, n
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), c(:,:), d(:,:), tau(:),      &
                                          wi(:), work(:), wr(:), z(:,:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F08NFF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n
       lda = n
       ldz = n
       ldc = n
       ldd = n
       lwork = 64*(n-1)
       ALLOCATE (a(lda,n),c(ldc,n),d(ldd,n),tau(n),wi(n),work(lwork),wr(n), &
          z(ldz,n))

!      Read A from data file

       READ (nin,*) (a(i,1:n),i=1,n)

!      Copy A into D.
       d(1:n,1:n) = a(1:n,1:n)

       WRITE (nout,*)
       FLUSH (nout)

!      Print Matrix A
!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A',ifail)

       WRITE (nout,*)
       FLUSH (nout)

!      Reduce A to upper Hessenberg form H = (Q**T)*A*Q
!      The NAG name equivalent of dgehrd is f08nef
       CALL dgehrd(n,1,n,a,lda,tau,work,lwork,info)

!      Copy A into Z
       z(1:n,1:n) = a(1:n,1:n)

!      Form Q explicitly, storing the result in Z
!      The NAG name equivalent of dorghr is f08nff
       CALL dorghr(n,1,n,z,ldz,tau,work,lwork,info)

!      Calculate the Schur factorization of H = Y*T*(Y**T) and form
!      Q*Y explicitly, storing the result in Z

!      Note that A = Z*T*(Z**T), where Z = Q*Y
!      The NAG name equivalent of dhseqr is f08pef
       CALL dhseqr('Schur form','Vectors',n,1,n,a,lda,wr,wi,z,ldz,work,lwork, &
          info)

!      Compute A - Z*T*Z^T from the factorization of A and store in matrix D.
!      The NAG name equivelent of dgemm is f06yaf.
       alpha = 1.0_nag_wp
       beta = 0.0_nag_wp
       CALL dgemm('N','N',n,n,n,alpha,z,ldz,a,lda,beta,c,ldc)
       alpha = -1.0_nag_wp
       beta = 1.0_nag_wp
       CALL dgemm('N','T',n,n,n,alpha,c,ldc,z,ldz,beta,d,ldd)

!      Find norm of difference matrix D and print warning if it is too large;
!      f06raf is the NAG name equivalent of the LAPACK auxiliary dlange
       norm = dlange('O',ldd,n,d,ldd,work)
       IF (norm>x02ajf()**0.8_nag_wp) THEN
          WRITE (nout,*) 'Norm of A-(Z*T*Z^T) is much greater than 0.'
          WRITE (nout,*) 'Schur factorization has failed.'
       ELSE
!         Print eigenvalues.
          WRITE (nout,*) 'Eigenvalues'
          WRITE (nout,99999)(' (',wr(i),',',wi(i),')',i=1,n)
       END IF

99999  FORMAT (1X,A,F8.4,A,F8.4,A)

    END PROGRAM f08nffe
