    PROGRAM f08vefe

!      F08VEF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dggsvp, f06raf, nag_wp, x02ajf, x04cbf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: eps, tola, tolb
       INTEGER                         :: i, ifail, info, irank, k, l, lda,    &
                                          ldb, ldq, ldu, ldv, m, n, p
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), b(:,:), q(:,:), tau(:),      &
                                          u(:,:), v(:,:), work(:)
       INTEGER, ALLOCATABLE            :: iwork(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Intrinsic Functions ..
       INTRINSIC                          max, real
!      .. Executable Statements ..
       WRITE (nout,*) 'F08VEF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)

!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) m, n, p
       lda = m
       ldb = p
       ldq = n
       ldu = m
       ldv = p
       ALLOCATE (a(lda,n),b(ldb,n),q(ldq,n),tau(n),u(ldu,m),v(ldv,p), &
          work(m+3*n+p),iwork(n))

!      Read the m by n matrix A and p by n matrix B from data file

       READ (nin,*) (a(i,1:n),i=1,m)
       READ (nin,*) (b(i,1:n),i=1,p)

!      Compute tola and tolb as
!          tola = max(m,n)*norm(A)*macheps
!          tolb = max(p,n)*norm(B)*macheps

       eps = x02ajf()
       tola = real(max(m,n),kind=nag_wp)*f06raf('One-norm',m,n,a,lda,work)*eps
       tolb = real(max(p,n),kind=nag_wp)*f06raf('One-norm',p,n,b,ldb,work)*eps

!      Compute the factorization of (A, B)
!          (A = U*S*(Q**T), B = V*T*(Q**T))

!      The NAG name equivalent of dggsvp is f08vef
       CALL dggsvp('U','V','Q',m,p,n,a,lda,b,ldb,tola,tolb,k,l,u,ldu,v,ldv,q, &
          ldq,iwork,tau,work,info)

!      Print solution

       irank = k + l
       WRITE (nout,*) 'Numerical rank of (A**T B**T)**T (K+L)'
       WRITE (nout,99999) irank

       WRITE (nout,*)
       FLUSH (nout)
       IF (m>=irank) THEN

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04cbf('Upper','Non-unit',irank,irank,a(1,n-irank+1),lda, &
             '1P,E12.4','Upper triangular matrix S','Integer',rlabs,'Integer', &
             clabs,80,0,ifail)

       ELSE

          ifail = 0
          CALL x04cbf('Upper','Non-unit',m,irank,a(1,n-irank+1),lda, &
             '1P,E12.4','Upper trapezoidal matrix S','Integer',rlabs, &
             'Integer',clabs,80,0,ifail)

       END IF
       WRITE (nout,*)
       FLUSH (nout)

       ifail = 0
       CALL x04cbf('Upper','Non-unit',l,l,b(1,n-l+1),ldb,'1P,E12.4', &
          'Upper triangular matrix T','Integer',rlabs,'Integer',clabs,80,0, &
          ifail)

       WRITE (nout,*)
       FLUSH (nout)

       ifail = 0
       CALL x04cbf('General',' ',m,m,u,ldu,'1P,E12.4','Orthogonal matrix U', &
          'Integer',rlabs,'Integer',clabs,80,0,ifail)

       WRITE (nout,*)
       FLUSH (nout)

       ifail = 0
       CALL x04cbf('General',' ',p,p,v,ldv,'1P,E12.4','Orthogonal matrix V', &
          'Integer',rlabs,'Integer',clabs,80,0,ifail)

       WRITE (nout,*)
       FLUSH (nout)

       ifail = 0
       CALL x04cbf('General',' ',n,n,q,ldq,'1P,E12.4','Orthogonal matrix Q', &
          'Integer',rlabs,'Integer',clabs,80,0,ifail)

99999  FORMAT (1X,I5)
    END PROGRAM f08vefe
